@@ -5,6 +5,7 @@ use cairo::{self, MatrixTrait};
 use attributes::Attribute;
 use coord_units::CoordUnits;
 use drawing_ctx::DrawingCtx;
+use error::RenderingError;
 use handle::RsvgHandle;
 use node::{NodeResult, NodeTrait, RsvgNode};
 use parsers::parse;
@@ -32,7 +33,7 @@ impl NodeClipPath {
         node: &RsvgNode,
         affine_before_clip: &cairo::Matrix,
         draw_ctx: &mut DrawingCtx,
-    ) {
+    ) -> Result<(), RenderingError> {
         let cascaded = node.get_cascaded_values();
 
         let clip_units = self.units.get();
@@ -43,7 +44,7 @@ impl NodeClipPath {
             if orig_bbox.rect.is_none() {
                 // The node being clipped is empty / doesn't have a
                 // bounding box, so there's nothing to clip!
-                return;
+                return Ok(());
             }
 
             let rect = orig_bbox.rect.unwrap();
@@ -60,7 +61,7 @@ impl NodeClipPath {
         cr.set_matrix(child_matrix);
 
         // here we don't push a layer because we are clipping
-        node.draw_children(&cascaded, draw_ctx, true);
+        let res = node.draw_children(&cascaded, draw_ctx, true);
 
         cr.set_matrix(save_affine);
 
@@ -72,6 +73,8 @@ impl NodeClipPath {
 
         let cr = draw_ctx.get_cairo_context();
         cr.clip();
+
+        res
     }
 }
 
