@@ -210,13 +210,13 @@ impl Filter for ColorMatrix {
 
                 let new_alpha = clamp(new_pixel_vec[3], 0.0, 1.0);
 
-                let premultiply = |x: f64| (clamp(x, 0.0, 1.0) * new_alpha * 255f64).round() as u8;
+                let premultiply = |x: f64| ((clamp(x, 0.0, 1.0) * new_alpha * 255f64) + 0.5) as u8;
 
                 let output_pixel = Pixel {
                     r: premultiply(new_pixel_vec[0]),
                     g: premultiply(new_pixel_vec[1]),
                     b: premultiply(new_pixel_vec[2]),
-                    a: (new_alpha * 255f64).round() as u8,
+                    a: ((new_alpha * 255f64) + 0.5) as u8,
                 };
 
                 output_data.set_pixel(output_stride, output_pixel, x, y);
