@@ -14,6 +14,7 @@ use property_bag::PropertyBag;
 use util::clamp;
 
 /// A light source node (`feDistantLight`, `fePointLight` or `feSpotLight`).
+#[derive(Clone)]
 pub enum LightSource {
     Distant {
         azimuth: Cell<f64>,
@@ -36,6 +37,23 @@ pub enum LightSource {
     },
 }
 
+/// A light source node with affine transformations applied.
+pub enum TransformedLightSource {
+    Distant {
+        azimuth: f64,
+        elevation: f64,
+    },
+    Point {
+        origin: Vector3<f64>,
+    },
+    Spot {
+        origin: Vector3<f64>,
+        direction: Vector3<f64>,
+        specular_exponent: f64,
+        limiting_cone_angle: Option<f64>,
+    },
+}
+
 impl LightSource {
     /// Constructs a new `feDistantLight` with empty properties.
     #[inline]
@@ -71,35 +89,71 @@ impl LightSource {
         }
     }
 
+    /// Returns a `TransformedLightSource` according to the given `FilterContext`.
+    #[inline]
+    pub fn transform(&self, ctx: &FilterContext) -> TransformedLightSource {
+        match self {
+            LightSource::Distant { azimuth, elevation } => TransformedLightSource::Distant {
+                azimuth: azimuth.get(),
+                elevation: elevation.get(),
+            },
+            LightSource::Point { x, y, z } => {
+                let (x, y) = ctx.paffine().transform_point(x.get(), y.get());
+                let z = ctx.transform_dist(z.get());
+
+                TransformedLightSource::Point {
+                    origin: Vector3::new(x, y, z),
+                }
+            }
+            LightSource::Spot {
+                x,
+                y,
+                z,
+                points_at_x,
+                points_at_y,
+                points_at_z,
+                specular_exponent,
+                limiting_cone_angle,
+            } => {
+                let (x, y) = ctx.paffine().transform_point(x.get(), y.get());
+                let z = ctx.transform_dist(z.get());
+                let (points_at_x, points_at_y) = ctx
+                    .paffine()
+                    .transform_point(points_at_x.get(), points_at_y.get());
+                let points_at_z = ctx.transform_dist(points_at_z.get());
+
+                let origin = Vector3::new(x, y, z);
+                let mut direction = Vector3::new(points_at_x, points_at_y, points_at_z) - origin;
+                let _ = direction.try_normalize_mut(0.0);
+
+                TransformedLightSource::Spot {
+                    origin,
+                    direction,
+                    specular_exponent: specular_exponent.get(),
+                    limiting_cone_angle: limiting_cone_angle.get(),
+                }
+            }
+        }
+    }
+}
+
+impl TransformedLightSource {
     /// Returns the unit (or null) vector from the image sample to the light.
     #[inline]
-    pub fn vector(&self, x: f64, y: f64, z: f64, ctx: &FilterContext) -> Vector3<f64> {
+    pub fn vector(&self, x: f64, y: f64, z: f64) -> Vector3<f64> {
         match self {
-            LightSource::Distant { azimuth, elevation } => {
-                let azimuth = azimuth.get().to_radians();
-                let elevation = elevation.get().to_radians();
+            TransformedLightSource::Distant { azimuth, elevation } => {
+                let azimuth = azimuth.to_radians();
+                let elevation = elevation.to_radians();
                 Vector3::new(
                     azimuth.cos() * elevation.cos(),
                     azimuth.sin() * elevation.cos(),
                     elevation.sin(),
                 )
             }
-            LightSource::Point {
-                x: light_x,
-                y: light_y,
-                z: light_z,
-            }
-            | LightSource::Spot {
-                x: light_x,
-                y: light_y,
-                z: light_z,
-                ..
-            } => {
-                let (light_x, light_y) =
-                    ctx.paffine().transform_point(light_x.get(), light_y.get());
-                let light_z = ctx.transform_dist(light_z.get());
-
-                let mut v = Vector3::new(light_x - x, light_y - y, light_z - z);
+            TransformedLightSource::Point { origin }
+            | TransformedLightSource::Spot { origin, .. } => {
+                let mut v = origin - Vector3::new(x, y, z);
                 let _ = v.try_normalize_mut(0.0);
                 v
             }
@@ -112,50 +166,27 @@ impl LightSource {
         &self,
         lighting_color: cssparser::RGBA,
         light_vector: Vector3<f64>,
-        ctx: &FilterContext,
     ) -> cssparser::RGBA {
         match self {
-            LightSource::Spot {
-                x: light_x,
-                y: light_y,
-                z: light_z,
-                points_at_x,
-                points_at_y,
-                points_at_z,
+            TransformedLightSource::Spot {
+                direction,
                 specular_exponent,
                 limiting_cone_angle,
                 ..
             } => {
-                let (light_x, light_y) =
-                    ctx.paffine().transform_point(light_x.get(), light_y.get());
-                let light_z = ctx.transform_dist(light_z.get());
-                let (points_at_x, points_at_y) = ctx
-                    .paffine()
-                    .transform_point(points_at_x.get(), points_at_y.get());
-                let points_at_z = ctx.transform_dist(points_at_z.get());
-
-                let mut s = Vector3::new(
-                    points_at_x - light_x,
-                    points_at_y - light_y,
-                    points_at_z - light_z,
-                );
-                if s.try_normalize_mut(0.0).is_none() {
-                    return cssparser::RGBA::transparent();
-                }
-
-                let minus_l_dot_s = -light_vector.dot(&s);
+                let minus_l_dot_s = -light_vector.dot(&direction);
                 if minus_l_dot_s <= 0.0 {
                     return cssparser::RGBA::transparent();
                 }
 
-                if let Some(limiting_cone_angle) = limiting_cone_angle.get() {
+                if let Some(limiting_cone_angle) = limiting_cone_angle {
                     if minus_l_dot_s < limiting_cone_angle.to_radians().cos() {
                         return cssparser::RGBA::transparent();
                     }
                 }
 
-                let factor = minus_l_dot_s.powf(specular_exponent.get());
-                let compute = |x| clamp(f64::from(x) * factor, 0.0, 255.0).round() as u8;
+                let factor = minus_l_dot_s.powf(*specular_exponent);
+                let compute = |x| (clamp(f64::from(x) * factor, 0.0, 255.0) + 0.5) as u8;
 
                 cssparser::RGBA {
                     red: compute(lighting_color.red),
