@@ -1,5 +1,6 @@
 use std::cell::{Cell, RefCell};
 use std::ops::Deref;
+use std::time::Instant;
 
 use cairo;
 use owning_ref::RcRef;
@@ -254,7 +255,18 @@ pub fn render(
     filter_node
         .children()
         // Skip nodes in error.
-        .filter(|c| !c.is_in_error())
+        .filter(|c| {
+            let in_error = c.is_in_error();
+
+            if in_error {
+                rsvg_log!(
+                    "(ignoring filter primitive {} because it is in error)",
+                    c.get_human_readable_name()
+                );
+            }
+
+            !in_error
+        })
         // Check if the node wants linear RGB.
         .map(|c| {
             let linear_rgb = {
@@ -265,66 +277,78 @@ pub fn render(
             };
 
             (c, linear_rgb)
-        })
-        .filter_map(|(c, linear_rgb)| {
-            let rr = RcRef::new(c).try_map(|c| {
-                // Go through the filter primitives and see if the node is one of them.
-                #[inline]
-                fn as_filter<T: Filter>(x: &T) -> &Filter {
-                    x
-                }
+        }).filter_map(|(c, linear_rgb)| {
+            let rr = RcRef::new(c)
+                .try_map(|c| {
+                    // Go through the filter primitives and see if the node is one of them.
+                    #[inline]
+                    fn as_filter<T: Filter>(x: &T) -> &Filter {
+                        x
+                    }
 
-                // Unfortunately it's not possible to downcast to a trait object. If we could
-                // attach arbitrary data to nodes it would really help here. Previously that
-                // arbitrary data was in form of RsvgCNodeImpl, but that was heavily tuned for
-                // storing C pointers with all subsequent downsides.
-                macro_rules! try_downcasting_to_filter {
-                    ($c:expr; $($t:ty),+$(,)*) => ({
-                        let mut filter = None;
-                        $(
-                            filter = filter.or_else(|| $c.get_impl::<$t>().map(as_filter));
-                        )+
-                        filter
-                    })
-                }
+                    // Unfortunately it's not possible to downcast to a trait object. If we could
+                    // attach arbitrary data to nodes it would really help here. Previously that
+                    // arbitrary data was in form of RsvgCNodeImpl, but that was heavily tuned for
+                    // storing C pointers with all subsequent downsides.
+                    macro_rules! try_downcasting_to_filter {
+                        ($c:expr; $($t:ty),+$(,)*) => ({
+                            let mut filter = None;
+                            $(
+                                filter = filter.or_else(|| $c.get_impl::<$t>().map(as_filter));
+                            )+
+                            filter
+                        })
+                    }
 
-                let filter = try_downcasting_to_filter!(
-                    c;
-                    blend::Blend,
-                    color_matrix::ColorMatrix,
-                    component_transfer::ComponentTransfer,
-                    composite::Composite,
-                    convolve_matrix::ConvolveMatrix,
-                    displacement_map::DisplacementMap,
-                    flood::Flood,
-                    gaussian_blur::GaussianBlur,
-                    image::Image,
-                    light::lighting::Lighting,
-                    merge::Merge,
-                    morphology::Morphology,
-                    offset::Offset,
-                    tile::Tile,
-                    turbulence::Turbulence,
-                );
-                filter.ok_or(())
-            }).ok();
+                    let filter = try_downcasting_to_filter!(c;
+                        blend::Blend,
+                        color_matrix::ColorMatrix,
+                        component_transfer::ComponentTransfer,
+                        composite::Composite,
+                        convolve_matrix::ConvolveMatrix,
+                        displacement_map::DisplacementMap,
+                        flood::Flood,
+                        gaussian_blur::GaussianBlur,
+                        image::Image,
+                        light::lighting::Lighting,
+                        merge::Merge,
+                        morphology::Morphology,
+                        offset::Offset,
+                        tile::Tile,
+                        turbulence::Turbulence,
+                    );
+                    filter.ok_or(())
+                }).ok();
 
             rr.map(|rr| (rr, linear_rgb))
-        })
-        .for_each(|(rr, linear_rgb)| {
+        }).for_each(|(rr, linear_rgb)| {
             let mut render = |filter_ctx: &mut FilterContext| {
-                if let Err(_) = rr.render(rr.owner(), filter_ctx, draw_ctx)
+                if let Err(err) = rr
+                    .render(rr.owner(), filter_ctx, draw_ctx)
                     .and_then(|result| filter_ctx.store_result(result))
                 {
-                    // Filter::render() returned an error. Do nothing for now.
+                    rsvg_log!(
+                        "(filter primitive {} returned an error: {})",
+                        rr.owner().get_human_readable_name(),
+                        err
+                    );
                 }
             };
 
+            let start = Instant::now();
+
             if rr.is_affected_by_color_interpolation_filters() && linear_rgb {
                 filter_ctx.with_linear_rgb(render);
             } else {
                 render(&mut filter_ctx);
             }
+
+            let elapsed = start.elapsed();
+            rsvg_log!(
+                "(rendered filter primitive {} in\n    {} seconds)",
+                rr.owner().get_human_readable_name(),
+                elapsed.as_secs() as f64 + f64::from(elapsed.subsec_nanos()) / 1e9
+            );
         });
 
     filter_ctx
