@@ -1,3 +1,5 @@
+use glib::translate::*;
+use glib_sys;
 use libc;
 use std::collections::HashMap;
 
@@ -270,6 +272,7 @@ pub extern "C" fn rsvg_load_new_node(
     parent: *const RsvgNode,
     pbag: *const PropertyBag,
     defs: *mut RsvgDefs,
+    out_is_svg: *mut glib_sys::gboolean,
 ) -> *const RsvgNode {
     assert!(!raw_name.is_null());
     assert!(!pbag.is_null());
@@ -295,7 +298,6 @@ pub extern "C" fn rsvg_load_new_node(
         // Whenever we encounter a node we don't understand, represent it as a defs.
         // This is like a group, but it doesn't do any rendering of children.  The
         // effect is that we will ignore all children of unknown elements.
-        //
         None => &(true, create_defs as NodeCreateFn),
     };
 
@@ -306,11 +308,25 @@ pub extern "C" fn rsvg_load_new_node(
     let node = create_fn(id, class, parent);
     assert!(!node.is_null());
 
+    let is_svg = {
+        let n: &RsvgNode = unsafe { &*node };
+        match n.get_type() {
+            NodeType::Invalid => unreachable!(),
+            NodeType::Svg => true,
+            _ => false,
+        }
+    };
+
     if id.is_some() {
         let n = unsafe { &*node };
         defs.insert(id.unwrap(), n);
     }
 
+    assert!(!out_is_svg.is_null());
+    unsafe {
+        *out_is_svg = is_svg.to_glib();
+    }
+
     node
 }
 
@@ -339,3 +355,20 @@ pub extern "C" fn rsvg_load_set_node_atts(
 
     node.set_overridden_properties();
 }
+
+#[no_mangle]
+pub extern "C" fn rsvg_load_set_svg_node_atts(
+    handle: *const RsvgHandle,
+    raw_node: *const RsvgNode,
+) {
+    assert!(!raw_node.is_null());
+    let node: &RsvgNode = unsafe { &*raw_node };
+
+    if node.get_type() != NodeType::Svg {
+        return;
+    }
+
+    node.with_impl(|svg: &NodeSvg| {
+        svg.parse_style_attrs(node, handle);
+    });
+}
