@@ -129,8 +129,9 @@ pub trait NodeTrait: Downcast {
         _cascaded: &CascadedValues,
         _draw_ctx: &mut DrawingCtx,
         _clipping: bool,
-    ) {
+    ) -> Result<(), RenderingError> {
         // by default nodes don't draw themselves
+        Ok(())
     }
 }
 
@@ -182,8 +183,6 @@ pub struct Children {
     next_back: Option<Rc<Node>>,
 }
 
-// Keep this in sync with rsvg-private.h:RsvgNodeType
-#[repr(C)]
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd)]
 pub enum NodeType {
     Invalid = 0,
@@ -279,6 +278,14 @@ impl Node {
         self.class.as_ref().map(String::as_str)
     }
 
+    pub fn get_human_readable_name(&self) -> String {
+        format!(
+            "{:?} id={}",
+            self.node_type,
+            self.get_id().unwrap_or("None")
+        )
+    }
+
     pub fn get_transform(&self) -> Matrix {
         self.transform.get()
     }
@@ -367,7 +374,13 @@ impl Node {
             }
         }
 
-        *self.result.borrow_mut() = self.node_impl.set_atts(node, handle, pbag);
+        match self.node_impl.set_atts(node, handle, pbag) {
+            Ok(_) => (),
+            Err(e) => {
+                self.set_error(e);
+                return;
+            }
+        }
     }
 
     fn parse_conditional_processing_attributes(&self, pbag: &PropertyBag) -> Result<(), NodeError> {
@@ -417,20 +430,35 @@ impl Node {
         cascaded: &CascadedValues,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) {
-        if self.result.borrow().is_ok() {
+    ) -> Result<(), RenderingError> {
+        if !self.is_in_error() {
             let cr = draw_ctx.get_cairo_context();
             let save_affine = cr.get_matrix();
 
             cr.transform(self.get_transform());
 
-            self.node_impl.draw(node, cascaded, draw_ctx, clipping);
+            let res = self.node_impl.draw(node, cascaded, draw_ctx, clipping);
 
             cr.set_matrix(save_affine);
+
+            res
+        } else {
+            rsvg_log!(
+                "(not rendering element {} because it is in error)",
+                self.get_human_readable_name()
+            );
+
+            Ok(()) // maybe we should actually return a RenderingError::NodeIsInError here?
         }
     }
 
     pub fn set_error(&self, error: NodeError) {
+        rsvg_log!(
+            "(attribute error for {}:\n  {})",
+            self.get_human_readable_name(),
+            error
+        );
+
         *self.result.borrow_mut() = Err(error);
     }
 
@@ -438,10 +466,6 @@ impl Node {
         self.result.borrow().is_err()
     }
 
-    pub fn get_result(&self) -> NodeResult {
-        self.result.borrow().clone()
-    }
-
     pub fn with_impl<T, F, U>(&self, f: F) -> U
     where
         T: NodeTrait,
@@ -463,10 +487,16 @@ impl Node {
         cascaded: &CascadedValues,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) {
+    ) -> Result<(), RenderingError> {
         for child in self.children() {
-            draw_ctx.draw_node_from_stack(&CascadedValues::new(cascaded, &child), &child, clipping);
+            draw_ctx.draw_node_from_stack(
+                &CascadedValues::new(cascaded, &child),
+                &child,
+                clipping,
+            )?;
         }
+
+        Ok(())
     }
 
     pub fn children(&self) -> Children {
@@ -605,14 +635,6 @@ impl DoubleEndedIterator for Children {
     }
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_get_type(raw_node: *const RsvgNode) -> NodeType {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    node.get_type()
-}
-
 pub fn box_node(node: RsvgNode) -> *mut RsvgNode {
     Box::into_raw(Box::new(node))
 }
@@ -647,25 +669,6 @@ pub extern "C" fn rsvg_node_unref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
     ptr::null_mut()
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_is_same(
-    raw_node1: *const RsvgNode,
-    raw_node2: *const RsvgNode,
-) -> glib_sys::gboolean {
-    let is_same = if raw_node1.is_null() && raw_node2.is_null() {
-        true
-    } else if !raw_node1.is_null() && !raw_node2.is_null() {
-        let node1: &RsvgNode = unsafe { &*raw_node1 };
-        let node2: &RsvgNode = unsafe { &*raw_node2 };
-
-        Rc::ptr_eq(node1, node2)
-    } else {
-        false
-    };
-
-    is_same.to_glib()
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_node_add_child(raw_node: *mut RsvgNode, raw_child: *const RsvgNode) {
     assert!(!raw_node.is_null());
@@ -736,16 +739,6 @@ pub extern "C" fn rsvg_node_children_iter_next(
     }
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_root_node_cascade(raw_node: *const RsvgNode) {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    let values = ComputedValues::default();
-
-    node.cascade(&values)
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -799,40 +792,12 @@ mod tests {
         ));
 
         let ref1 = box_node(node);
+        let node1: &RsvgNode = unsafe { &*ref1 };
 
         let ref2 = rsvg_node_ref(ref1);
+        let node2: &RsvgNode = unsafe { &*ref2 };
 
-        assert!(rsvg_node_is_same(ref1, ref2) == true.to_glib());
-
-        rsvg_node_unref(ref1);
-        rsvg_node_unref(ref2);
-    }
-
-    #[test]
-    fn different_nodes_have_different_pointers() {
-        let node1 = Rc::new(Node::new(
-            NodeType::Path,
-            None,
-            None,
-            None,
-            ptr::null_mut(),
-            Box::new(TestNodeImpl {}),
-        ));
-
-        let ref1 = box_node(node1);
-
-        let node2 = Rc::new(Node::new(
-            NodeType::Path,
-            None,
-            None,
-            None,
-            ptr::null_mut(),
-            Box::new(TestNodeImpl {}),
-        ));
-
-        let ref2 = box_node(node2);
-
-        assert!(rsvg_node_is_same(ref1, ref2) == false.to_glib());
+        assert!(Rc::ptr_eq(node1, node2));
 
         rsvg_node_unref(ref1);
         rsvg_node_unref(ref2);
