@@ -323,7 +323,7 @@ rsvg_cairo_check (gconstpointer data)
 						 dimensions.height + 2 * FRAME_SIZE);
     cr = cairo_create (render_surface);
     cairo_translate (cr, FRAME_SIZE, FRAME_SIZE);
-    rsvg_handle_render_cairo (rsvg, cr);
+    g_assert (rsvg_handle_render_cairo (rsvg, cr));
 
     surface_a = extract_rectangle (render_surface,
 				   FRAME_SIZE,
